/*******************************************************************************************************************************
** FILE: package.h - packages and database specification
** DATE: 2011.02.07 DJM
** DESC: defines the detailed operating specification for reading and writing and updating celestron package data
**
** NOTE: Copywrite (c) 2009 by Celestron. 
********************************************************************************************************************************
** Last Modified:
        2011.12.02 DCD - Added PKG_PLATFORM_MC5 and FontInfoTy
        2011.11.10 DCD - corrected errors in the language definitions.
        2011.06.23 DCD - Added FileInfoTy (used only by CFM and Build tools)
        2011.04.11 DCD - Renamed certain enumerations for consistency
                         Added new PLATFORM enumertion (subsumes TARGET)
                         Updated BootInfoTy
                         Added BootUpdaterInfoTy and TocInfoTy
                         Added NumOptionalPackages to AppInfo
                         Added PkgFlags(dependancy/localization) to PkgDescriptorTy, TOCEntryTy, PkgInfo
                         Added Lang to PkgInfoTy
**      2011.02.07 DJM  -Added definition for FONT package
                        -Added NumRequiredPkgsUB to AppInfoTy
                        -Added PkgDescriptorTy for application to enforce min versions of packages
                        -Added TocEntryTy for use in accessing table of contents
                        -Added PKG_TYPE_TOC for the table of contents package
*******************************************************************************************************************************/


#ifndef __PACKAGE_H__
#define __PACKAGE_H__
//==============================================================================================================================
// Celestron Firmware Update, Package Structure, Definitions, and Other Conceptual Details
//==============================================================================================================================
//
// Package Image:
//              The smallest downloadable "unit" that will be downloaded from the firmware update program, it can contain an
//              application or bootloader firmware image, or one or more database images. For a package that contains more than
//              one database item, these will generally be associated with a common set based on the type of data it contains,
//              e.g., categories such as object data, menus, screen text, audio files which may contain multiple database items.
//              The data can also be a set of different language versions of a given database such as for menus or screen text.
//              There are no specific limitations on how databases are presented, combined, or located within a target device,
//              and the approach should be tailored for each target implementation. The concepts should, however, be consistent
//              between implementations once they are established.
//
//              On the PC, a package is generated by a target-specific tool that is designed to take source information for the
//              application or boot firmware or database sets to then massage into a predefined format. The tool must generate
//              the proper data structure for a package as defined bellow, and will also generate a crc and do light encryption
//              of the data. This becomes the "Package Image" however, it is then further encapsulated as a "Package File" that
//              will include some additional information to be decoded by the PC firmware update program.
//
//              On the target the package image structure is integral to the bootloader, application, and database information
//              as it resides within the context of the target's code execution and access parameters. In essence, the various
//              structures within the package can be referenced by pointers to the package's pre-generated internal structures,
//              and any other structred data within the databases.
//
//              During the update process the internal structures are used to identify the interoperability of various packages
//              that might be presented to the target for updating, and *** must be validated within the target ***.
//
//              The structure of an application or boot package:
//              BASE: +0x0000           AppInfoTy
//                    +0x0020           PkgDescriptorTy for first required package
//                    +0x0025           PkgDescriptorTy for second required pacakage,
//                    +0x0020 + 0x05*N  PkgDescriptorTy for last req. package (N = NumRequiredPkgs in AppInfoTy)
//                    + ????            Application code, offset as per AppInfoTy Struct
//
//              BASE: +0x0000           BootInfoTy
//                    + ????            Boot code, offset as per BootInfoTy
//
//              The structure of a database package:
//              BASE: +0x0000           PkgInfoTy, which indicates number of databases among other things
//                    +0x0020           DbInfoTy, which indicates the type of Db, offset within package, language tag, etc.
//                    +0x0030           DbInfoTy second database info
//                    +0x0040 * N       DbInfoTy last database info
//                    +0x0010 * (N+2)   First Database
//                    +<second DB offs> Second Database
//
// Package File:
//              The Package Image represents packaged data as it resides within a target device as it is generated on a PC. The
//              Package File is the further encapsulated Package Image that contains information as firmware rev changes, crc,
//              minimum CFM version, and is more specific to the operation of the firmware update manager. It is essentially
//              stripped by the firmware update manager before being transmitted to the target.
//
// Collection:
//              One embodiment of a collection is the _complete_ set of packages that are required for various implementation
//              levels of a target device's operation. An example could be different versions of a hand control, such as for
//              German Equatorial or Alt-Azm operation, or different feature level implementations of the various databases and
//              languages within the collective packages. Another could be difference levels of feature implementations, such as
//              custom object data, menus, audio files, etc.
//
//              From a practical standpoint, the target device should have a list of packages that are required for the current
//              implementation that is operating on the target. The firmware update program can querry the target (in boot mode)
//              for the list of packages that are required. This "collection list" is the primary package info block, and can
//              reside in various memory locations (tbd 20100916), such as EEPROM, or on-chip or off-chip NVM. Collections
//
//
//              NOTES ABOUT APPLICATION UPDATE PROCESS:
//
//              1) firmware update tool gets a list of packages -- primary, alternate, expansion -- that are currently installed
//              on target device. Including the version information -- sanity check on minversion comparisons where necessary
//
//              2) firmware update tool sends the app, boot, or package info block of the package that it proposes to update or
//              to add to the target. The bootloader must then validate serveral aspects of the package info, the least of which
//              is that the package is appriate for the system, and that it meets minimum version requirements.
//
//              3) Once validated, the firmware update tool will negotiate transfer parameters, which includes limiting the
//              transfer size, or any other oparational limitations that may be needed for a give target.
//
//              4) The firmware update tool starts sending the package data to the target, and the target programs the appriate
//              memory location
//
//==============================================================================================================================
// The following are descriptions of the various structures that make up the packages
//==============================================================================================================================
//
// VersionTy:   The purpose of the VersionTy struct is to identify the revision of a given package firmware, or data. This
//              assists in the process of validating firmware updates based on minimum version requirements of the propsed new
//              package vs. existing packages on the target device.
//
// SystemTy:    The SystemTy struct is used to identify specific details of a given system for both reporting and validation
//              purposes during the update process. The group field identifies a technology group, or hardware platform, such
//              as the StarSense based CortexM3. The Platform is the particular implementation of the hardware technology, such
//              as the SkyProdigy, or GEM Pro versions of the StarSense HC technology.
//
// ImageInfoTy: The ImageInfoTy struct, by design, resides at the start of a package file, or memory image. It starts with a CRC
//              that is used to validate the integrity of the file / image by the target, during initialziation, or by the
//              firmware update tool in preparation of doing an update. It also contains the version and system information
//              structs.
//
// DbInfoTy:    A DbInfoTy struct identifies the specific database within a package. Because a package can contain one or more
//              databases, the db info struct is used to identify and locate a database within a memory image area that pertains
//              to a given package
//
// BootInfoTy:  The BootInfoTy Struct is a specific instance of a package that contains information about the bootloader as well
//              as the bootloader firmware. On the target, it defines the memory image footprint that is associated with the
//              bootloader, and as it contains an ImageInfoTy struct, it is used for reporting and validation of the bootloader
//              firmware during initialization or during an update. Aside from firmware image validation and identification, it
//              also contains the starting address for the primary application that runs on the target.
//
// AppInfoTy:   The AppInfoTy struct is a specific instance of a package that contains an application, with similar purpose as
//              the bootloader version of the struct. The app info struct also contains the address of for a table of pointers
//              to packages installed on the target, and this table is also considered the "primary package collection" for the
//              target.
//
//## TODO ##
//
// refine definitions and enumerations for various fields, such as for SystemTy.GroupUW, or SystemTy.PlatformUW. Also, determine
// the implications of this implementation for legacy products, etc.

//==============================================================================================================================
// standard header includes
//==============================================================================================================================
//#include <stm32f10x_lib.h>                                      // stm32f10x definitions
//#include <RTL.H>                                                // real-time lib
#include "datatypes.h"                                             // platform type and hardware specific definitions



//******************************************************************************************************************************
//******                                         global defines                                             ********
//******************************************************************************************************************************

// if package.h changes, the revision should be bumped so that build tools, and the application can enforce requirements and to
// propagate updates to various files that use package.h
#define PKG_CONFIG_REVISION_MAJ                 0x01            // package format revision
#define PKG_CONFIG_REVISION_MIN                 0x01            //
#define PKG_CONFIG_REVISION_BLD                 0x0001          //


// the following #defines are meant to standardize PkgInfoTy parameters for group, platform, target, variant and type

// PKG_PLATFORM declares the type of bootloader, but it also declares a family of compatilbe firmware. For Example,
// legacy CGE firmware and legacy CPC firmware can both be installed on any PKG_PLATFORM_MC2 motor board. even though
// they control different mounts.  Not all functions would work, but it the firmware would drive motors and not brick the mount.
//. By contrast CPC Deluxe firmware (which requires PKG_PLATFORM_MC4) would break an MC2 board if installed.

// motor control platforms
#define PKG_PLATFORM_MC2                        0x02   // PIC16 motor controls
#define PKG_PLATFORM_MC3                        0x03   // PIC18 with CGEpro style driver controls
#define PKG_PLATFORM_MC4                        0x04   // PIC18 motor controllers (except CGEpro)
#define PKG_PLATFORM_MC5                        0x05   // ARMxxx One chip solution

// HC platforms
#define PKG_PLATFORM_HC_LEGACY                  0x10   // PIC17 (GEM, NXS, SLT_LCM)
#define PKG_PLATFORM_HC_PLUS                    0x11   // PIC18 (gemPLUS, aamPLUS, smallPLUS,...)
#define PKG_PLATFORM_HC_PLUSAUDIO               0x12   // reserved for Audio HC
#define PKG_PLATFORM_HC_HCPRO                   0x13   // ARM32 (currently only AAMPRO)

// Accessory Platforms
#define PKG_PLATFORM_AIS                        0x20   // AIS camera based on Cortex
#define PKG_PLATFORM_WIFI                       0x21   //
#define PKG_PLATFORM_FOCUS                      0x23
#define PKG_PLATFORM_DEROTATOR                  0x24


// Package HCMODEL carries the type (varient) of the HC , e.g. GEM vs AAM vs Small DB.    This information is convolved
// with the differnt PLATFORMs available for HC's.  This is is redundant.
// Implementation Ordered List:
#define PKG_HCMODEL_AAMLEGACY                   0x01           // Alt-Azm Mount for legacy platform
#define PKG_HCMODEL_AAMLEGACYSMLDB              0x02           // Alt-Azm Mount for lagacy platform with small db
#define PKG_HCMODEL_GEMLEGACY                   0x03           // German Equatorial Mount for legacy platform
#define PKG_HCMODEL_AAMPLUS                     0x04           // alt-azm mount with PIC18 HC, or MCPP implemented
#define PKG_HCMODEL_GEMPLUS                     0x05           // GEM mount with PIC18 HC, or MCPP implemented
#define PKG_HCMODEL_AAMPRO                      0x06           // Alt-Azm Mount with HC Pro features
#define PKG_HCMODEL_GEMPRO                      0x07           // GEM with HC Pro features
#define PKG_HCMODEL_CUSTOM                      0xfe           //



// Package MCMODEL carries the mount variant e.g. CGE vs ASGT vs CPC.  Like the HC Models it is _currently_
//possible to infer platform from model.  But it may soon be possible to have firmware for SLT running on the
// legacy MC2 board, and a different version of the firmware for the SLT running on a onechip solution.(differnt platform).
// Implementation Ordered List:
#define PKG_MCMODEL_NOTSPECIFIED                0x00            // package is universal, or group specific
#define PKG_MCMODEL_NXSGPS                      0x01            // NexStar GPS -- the original          HCMODEL_AAMLEGACY
#define PKG_MCMODEL_NXSGPS_NONL                 0x02            // NexStar GPS w/o North and Level      HCMODEL_AAMLEGACY
#define PKG_MCMODEL_NXSi                        0x03            // NexStar 'i' series                   HCMODEL_AAMLEGACY
#define PKG_MCMODEL_NXSSE                       0x04            // NexStar Special Edition              HCMODEL_AAMLEGACY
#define PKG_MCMODEL_CGE                         0x05            // The original "CGE"                   HCMODEL_GEMLEGACY
#define PKG_MCMODEL_ASC                         0x06            // ASGT, CAT, CG5 Advanced              HCMODEL_GEMLEGACY
#define PKG_MCMODEL_SLT                         0x07            // SLT Series, alt-azm mount            HCMODEL_AAMLEGACYSMLDB
#define PKG_MCMODEL_LEGEND                      0x08            // CDK20 Mount, original "CGEPRO"       HCMODEL_GEMLEGACY
#define PKG_MCMODEL_CPC                         0x09            // The CPC Series                       HCMODEL_AAMLEGACY
#define PKG_MCMODEL_GT                          0x0A            // The Flash Upgradeable NexStar GT     HCMODEL_AAMSMALLDB
#define PKG_MCMODEL_NXS45_SE                    0x0B            // NexStar SE 4" and 5" model           HCMODEL_AAMLEGACY
#define PKG_MCMODEL_NXS68_SE                    0x0C            // NexStar SE 6" and 8" model           HCMODEL_AAMLEGACY
#define PKG_MCMODEL_CGEPRO                      0x0D            // CGE update, GEM mount                HCMODEL_GEMLEGACY (*)
#define PKG_MCMODEL_CGEM                        0x0E            // medium size GEM                      HCMODEL_GEMLEGACY (*)
#define PKG_MCMODEL_LCM                         0x0F            // "Low Cost Mount" alt-azm mount       HCMODEL_AAMSMALLDB
#define PKG_MCMODEL_SKYPRODIGY                  0x10            // SkyProdigy series                    HCMODEL_SKYSENSE
#define PKG_MCMODEL_CUSTOM                      0xfe            // custom applications                  (any)



// for completeness there should be a list (or several lists) for each of the accessory platforms
#define PKG_AISMODEL_SKYPRODIGY                 0x01


// the following list subsumed by the addtion of more detailed PLATFORM list above.
// package target defines the hardware / electronics that the package is targeted to
// Alpha Ordered List:
//#define PKG_TARGET_HC                           0x01            // Hand Controller
//#define PKG_TARGET_MC                           0x02            // Motor Controller (all)
//#define PKG_TARGET_MCPRI                        0x03            // Primary (Azm) Motor Controller
//#define PKG_TARGET_MCSEC                        0x04            // Secondary (Alt) Motor Controller
//#define PKG_TARGET_AIS                          0x05            // Alignment Imaging Camera
//#define PKG_TARGET_WIFI                         0x06            // Wifi Interface
//#define PKG_TARGET_CUSTOM                       0xfe            // undefined (uncatorgorized, special hardware)

// Variant describes the use/context of the firmware or data
// Priority Ordered List:
#define PKG_VARIANT_PRODUCTION                  0x01            // production (public) release
#define PKG_VARIANT_BETA                        0x02            // beta testing
#define PKG_VARIANT_TEST                        0x03            // testing (manufacturing, special purpose firmware)
#define PKG_VARIANT_PROTO                       0x04            // prototype
#define PKG_VARIANT_CUSTOM                      0xfe            // undefined, or other special firmware variant

// PKG_TYPE for SkyProdigy Hand Control
// "PKG_TYPE_nnnnnn" #defines package contents in terms of the type of data contained within the package. This value eventually
// becomes part of a DbId value that encompasses the the package type and db type along with an index value to an object within
// the database itself. NOTE: The valid range of this #define is 1-31, except 0xfe for custom.
// Alpha Ordered List:
#define PKG_TYPE_BOOTLOADER                     0x00            // Bootloader exectable
#define PKG_TYPE_BOOTUPDATER                    0x01            // executable with  bootloader package as payload
#define PKG_TYPE_APPLICATION                    0x02            // Application Firmware
#define PKG_TYPE_TOC                            0x03            // Package Table of Contents
#define PKG_TYPE_DBOBJ                          0x04            // Object database
#define PKG_TYPE_DBOBJLANGPACK                  0x05            // Language pack for DB objects
#define PKG_TYPE_LISA                           0x06            // LISA related databases
#define PKG_TYPE_OTHERFIRMWARE                  0x07            // Other (custom) Firmware
#define PKG_TYPE_SYSTEM                         0x08            // screen text, menus, etc.
#define PKG_TYPE_SYSTEMLANGPACK                 0x09            // Language pack for system text, menus, etc.
#define PKG_TYPE_SYSTEMFONT                     0x0A            // font data
#define PKG_TYPE_CUSTOM                         0xfe            // (always 0xfe)



//PKG_TYPE for PLUS HC's
#define PKG_TYPE_BOOTLOADER                     0x00            // Bootloader exectable
#define PKG_TYPE_BOOTUPDATER                    0x01            // executable with  bootloader package as payload
#define PKG_TYPE_APPLICATION                    0x02            // Application Firmware
#define PKG_TYPE_TOC                            0x03            // Package Table of Contents
#define PKG_TYPE_LARGEDB                        0x04            // SAO, NGC and other large DB's
#define PKG_TYPE_SMALLDB                        0x05            // Messier, double star, and other small DB
#define PKG_TYPE_ALLTEXT                        0x06            // all text for menus as well as DB objects (current style)
#define PKG_TYPE_SYSTEMTEXT                     0x07            // Menu's and system text only (not used yet)
#define PKG_TYPE_DBOBJECTTEXT                   0x08            // DB object test only (not used yet)
#define PKG_TYPE_FONT                           0x0A            // value chosen to match SkyProdigy



// Language identifier: ISO 639-1 Code
#define LANG_NA                                 0x00            // not applicable (no localizable data)
#define LANG_EN                                 0x01            // English
#define LANG_ES                                 0x02            // Spanish
#define LANG_FR                                 0x03            // French
#define LANG_DE                                 0x04            // Germain
#define LANG_IT                                 0x05            // Italian
#define LANG_RU                                 0x06            // Russian
#define LANG_FIRST_NON_ALPHABET                 0x20
#define LANG_ZHS                                0x20            // Chinese Simplified
#define LANG_ZHT                                0x21            // Chinese Traditional
#define LANG_JA                                 0x22            // Japanese
#define LANG_KO                                 0x23            // Korean
#define LANG_ALPHABET                           0xFC            // supports _all_ Alphabet based languages (currently RU,EN, + 4 Euro languages)
#define LANG_ALL                                0xFE            // package contains sub packages accounting for all langauges
#define LANG_RESERVED                           0xFF            // Used in firmware to indicate no language selected.


// The following DB lists are for SkyProdigy

// PKG_TYPE_DBOBJ (0x01) db types, NOTE: The valid range for DB_TYPE is 1-127:
#define DB_TYPE_NA                              0x00            // not applicable, or invalid
#define DB_TYPE_ASTERISMS                       0x01            // Asterisms
#define DB_TYPE_CALDWELL                        0x02            // Caldwell catalog
#define DB_TYPE_CONSTELLATIONS                  0x03            // Constellations
#define DB_TYPE_DOUBLESTARS                     0x04            // double stars
#define DB_TYPE_GALAXIES                        0x05            // galaxies
#define DB_TYPE_GLOBULARCLUSTERS                0x06            // globular clusters
#define DB_TYPE_MESSIER                         0x07            // messier catalog
#define DB_TYPE_NAMEDOBJECTS                    0x08            // named objects
#define DB_TYPE_NAMEDSTARS                      0x09            // named stars
#define DB_TYPE_NEBULAE                         0x0A            // nebulae
#define DB_TYPE_NGC                             0x0B            // NGC catalog
#define DB_TYPE_OPENCLUSTERS                    0x0C            // open clustsers
#define DB_TYPE_PLANETARYNEBULAE                0x0D            // planetary nebulae
#define DB_TYPE_SOLARSYSTEM                     0x0E            // planets sun and moon catalog (text and desc. only)
#define DB_TYPE_SAO                             0x0F            // SAO catalog
#define DB_TYPE_SKYTOUR                         0x10            // sky tour objects
#define DB_TYPE_VARIABLESTARS                   0x11            // variable stars

// PKG_TYPE_SYSTEM (0x02) db types, NOTE: The valid range for DB_TYPE is 1-127:
#define DB_TYPE_MENUITEMS                       0x01            // menu items used in menuing
#define DB_TYPE_MISCTEXT                        0x02            // misc. text items
#define DB_TYPE_SCREENTEXT                      0x03            // screen text items used for generating screens
#define DB_TYPE_USERMENU                        0x04            // user generated menus
#define DB_TYPE_VENDORMENU                      0x05            // vendor specific menus

// PKG_TYPE_LISA (0x05) db types, NOTE: The valid range for DB_TYPE is 1-127:
#define DB_TYPE_LISAAPM                         0x01            // LISA APM Database
#define DB_TYPE_LISAQUAD                        0x02            // LISA Quad Database

// PKG_TYPE_TOC (0x0a)  db types, NOTE: the valid range for DB_TYPE is 1-127
#define DB_TYPE_TOC                             0x01            //





//******************************************************************************************************************************
//******                                         global typedefs                                            ********
//******************************************************************************************************************************


typedef struct VersionTg {                             // version information struct for each
  UWORD                                                 // 2    //   type of image in memory
    BuildUW;                                                    // build -- typically julian day
  UBYTE                                                 // 1    //
    MinorUB;                                                    // minor part of version
  UBYTE                                                 // 1    //
    MajorUB;                                                    // major part of version
} VersionTy;                                            //   4  // size of this object, must be padded to 4 byte boundry


typedef struct SystemTg {
  UBYTE                                                 // 1    //
    PlatformUB;                                                  // hardware definition
  UBYTE                                                 // 1    //
    ModelUB;                                                    // HC model, or MC model, or accessory Model
  UBYTE                                                 // 1    //
    reserved;                                                   //
  UBYTE                                                 // 1    //
    VariantUB;                                                  // production, release, beta, etc.
} SystemTy;                                             // 4  //




typedef struct ImageInfoTg {                           // effectively at the beginning of all images in memory or in
  UDWRD                                                 // 4    //   files, as part of a the boot, app, or package info block
    CrcUD;                                                      // CRC for image (the 2MSB's are reserved for future expansion)
  UDWRD                                                 // 4    //
    LengthUD;                                                   //
  VersionTy                                             // 4    //
    VersionSt;                                                  // version of associated image (boot, app, db, etc.
  SystemTy                                              // 4    //
    SystemSt;                                                   // system information pertaining to the image
} ImageInfoTy;                                          //   16 // size of this object, must be padded to 4 byte boundry



typedef struct BootInfoTg {                            // structure at beginning of boot image
  ImageInfoTy                                           // 16    //
    ImageInfoSt;                                                // bootloader's image info block
  VersionTy                                             // 4    //
    MinCfmVersionSt;                                            // minimum version of CFM to be used
  VersionTy                                             // 4    //
    MinAppVersionSt;                                            // minimum version of application
  UDWRD                                                 // 4    //
    AppInfoAddrUD;                                              // absolute starting address of application's info block
  UBYTE                                                 // 4   //
    MinTocVersionUD;                                             // a hint to CFM not strictly required???
} BootInfoTy;                                           //   32 // size of this object, must be padded to 4 byte boundry

typedef struct BootUpdaterInfoTg {
  ImageInfoTy                                           // 16   //
    ImageInfoSt;                                                // application's image info block
  VersionTy                                             // 4    //
    MinCfmVersionSt;                                            // copied from payload bootloader
  VersionTy                                             // 4    //
    MinAppVersionSt;                                            // copied from payload bootlaoder
  UDWRD                                                 // 4    //
    PayloadStartAddrUD;                                         // absolute address of payload bootloader installed by build tool
  UDWRD                                                 // 4   //
    PayloadLengthUD;                                            //
} BootUpdaterInfoTy;                                    // 32   //

typedef struct AppInfoTg {                             // structure at beginning of app image
  ImageInfoTy                                           // 16   //
    ImageInfoSt;                                                // application's image info block
  VersionTy                                             // 4    //
    MinBootVersionSt;                                           // minimum version of bootloader
  UDWRD                                                 // 4    //
    AppStartAddrUD;                                             // absolute starting address of application (platform specific)
  UDWRD                                                 // 4    //
    TocPkgAddrUD;                                               // absolute starting address of packages info block
  UBYTE                                                 // 1    //
    NumRequiredPkgsUB;                                          //
  UBYTE                                                 // 1    //
    NumOptionalPkgsUB;                                          //
  UBYTE                                                 // 2    //
    Rsvd[2];                                                    //
} AppInfoTy;                                            //   32 // size of this object, must be padded to 4 byte boundry

typedef struct PkgDescriptorTg {                       // this structure used as entries in a "packages reqruired"
                                                                //array that is part of the app info area of the application
  UBYTE                                                 // 1    //
    PkgTypeUB;                                                  // package type
  UBYTE                                                 // 1   //
    PkgFlagsUB;                                                 // flags for dependancy and localization
  UBYTE                                                 // 2   //
    Rsvd[2];
  VersionTy                                             // 4    //
    MinVersionSt;                                               // minimum version requirement for the package (as per the app)
} PkgDescriptorTy;                                      //   8  //


typedef struct PkgInfoTg {                             // structure at beginning of Database package image
  ImageInfoTy                                           // 16   //
    ImageInfoSt;                                                // package' image info block
  VersionTy                                             // 4    //
    MinAppVersionSt;                                            // minimum version of application
  UBYTE                                                 // 1    //
    PkgTypeUB;                                                  // enumeration of package type, e.g., objects, menus, text
  UBYTE                                                 // 1   //
    PkgFlagsUB;                                                 // flags for interpackage dependancy and localization
  UBYTE                                                 // 1    //
    PkgLang;                                                    // number of databased included in package
  UBYTE                                                 // 1    //
    NumDatabasesUB;                                             // number of databased included in package
  UBYTE                                                 // 8   //
    Rsvd[8];                                                   //
} PkgInfoTy;                                            //   32 // size of this object, must be padded to 4 byte boundry


typedef struct TocInfoTg {                             // structure at beginning of TOC package image
  ImageInfoTy                                           // 16   //
    ImageInfoSt;                                                // package' image info block
  VersionTy                                             // 4    //
    MinAppVersionSt;                                            // minimum version of application
  UBYTE                                                 // 1    //
    PkgTypeUB;                                                  // enumeration of package type, e.g., objects, menus, text
  UBYTE                                                 // 1
    NumRequiredPkgsUB;                                          //
  UBYTE                                                 // 1
    NumOptionalPkgsUB;                                          //
  UBYTE                                                 // 10   //
    Rsvd[9];                                                   //
} TocInfoTy;                                            //   32 // size of this object, must be padded to 4 byte boundry


typedef struct TocEntryTg {                            // structure comprises entries in a TOC (array)
  UBYTE                                                 // 1    //
    PkgTypeUB;                                                  // package type
  UBYTE                                                 // 1   //
    PkgFlagsUB;
  UBYTE
    Rsvd[2];
  UDWRD                                                 // 4    //
    PkgAddrUD;                                                  // absolute address of package object
} TocEntryTy;                                           //   8 //


typedef struct DbInfoTg {                              // a descriptor for each database within a package -- used to
  UDWRD                                                 // 4    //
    OffsetUD;                                                   // offset from start of package to database entry
  UDWRD                                                 // 4    //
    LengthUD;                                                   // length of database
  UBYTE                                                 // 1    //
    DbLangUB;                                                   // enumeration of language identifier
  UBYTE                                                 // 1    //
    DbTypeUB;                                                   // enumeration of type of database identifier
  UBYTE                                                 // 6    //
    Rsvd[6];                                                    //
} DbInfoTy;                                             //   16 //

typedef struct FontInfoTg {                             // a block of data at the front of a font database (after the 32 byte pkg header)
  UWORD  Size;                                            // 2 // the number of characters in the font
  UBYTE  Width;                                           // 1 // the width of a character (in pixels)
  UBYTE  Height;                                          // 1 // the height of the character (in pixels) 
  UBYTE  Rsvd[12];
} FontInfoTy;

typedef struct FileInfoTg {
  UWORD         Rsvd1;//0xFFFF;                         //  2   // must be 0xFFFF marks the end of the plain text headers
  UWORD         Crc;                                    //  2   // CRC of the encrypted file for validating after the download
  VersionTy     MinCFMVersion;                          //  4   // determines the min version of the PC Loader for this file
  UDWRD         StartAddress;                           //  4   // the destination address of the package in FLASH Memory.
  UWORD         PackageType;                            //  2   // one of {BOOTLOADER, UPDATER, APPLICATION, PKG(generic DB pkg))
  UWORD         Rsvd2;                                  //  2   // reserved for future use
} FileInfoTy;                                           // 16   //


//******************************************************************************************************************************
//******                                     exports of variables and functions                                         ********
//******************************************************************************************************************************

#endif //#ifndef __PACKAGE_H__
